@@ -154,6 +154,7 @@ static NV_STATUS _issueRpcAndWait(OBJGPU *pGpu, OBJRPC *pRpc)
 
     // should not be called in broadcast mode
     NV_ASSERT_OR_RETURN(!gpumgrGetBcEnabledStatus(pGpu), NV_ERR_INVALID_STATE);
+    NV_CHECK(LEVEL_ERROR, rmDeviceGpuLockIsOwner(pGpu->gpuInstance));
 
     if (bProfileRPC)
     {
@@ -1351,6 +1352,10 @@ NV_STATUS rpcGspSetSystemInfo_v17_00
         OBJTMR *pTmr = GPU_GET_TIMER(pGpu);
         rpcInfo->sysTimerOffsetNs = pTmr->sysTimerOffsetNs;
 
+
+        // Indicate whether the driver supports NV2080_NOTIFIERS_UCODE_RESET event.
+        rpcInfo->bTdrEventSupported = pGpu->getProperty(pGpu, PDB_PROP_GPU_SUPPORTS_TDR_EVENT);
+
         status = _issueRpcAsync(pGpu, pRpc);
     }
 
@@ -1586,7 +1591,7 @@ NV_STATUS rpcRmApiControl_GSP
                 GPU_LOCK_FLAGS_SAFE_LOCK_UPGRADE, RM_LOCK_MODULES_RPC, &gpuMaskRelease));
     }
 
-    rmapiutilGetControlInfo(cmd, &ctrlFlags, &ctrlAccessRight);
+    rmapiutilGetControlInfo(cmd, &ctrlFlags, &ctrlAccessRight, NULL);
     bCacheable = rmapiControlIsCacheable(ctrlFlags, ctrlAccessRight, NV_TRUE);
 
     pCallContext = resservGetTlsCallContext();
@@ -2003,3 +2008,27 @@ done:
     }
     return status;
 }
+
+/*
+ * Sends ack from CPU-RM to GSP-RM that ECC error
+ * notifier write has completed.
+ */
+NV_STATUS rpcEccNotifierWriteAck_v23_05
+(
+    OBJGPU                *pGpu,
+    OBJRPC                *pRpc
+)
+{
+    NV_STATUS status = NV_ERR_NOT_SUPPORTED;
+
+    if (IS_GSP_CLIENT(pGpu))
+    {
+        status = rpcWriteCommonHeader(pGpu, pRpc, NV_VGPU_MSG_FUNCTION_ECC_NOTIFIER_WRITE_ACK, 0);
+        if (status != NV_OK)
+            return status;
+
+        status = _issueRpcAsync(pGpu, pRpc);
+    }
+
+    return status;
+}
